@@ -118,6 +118,11 @@ class Order:
 
         self.related_orders = None
 
+        # Reduce only order
+        self.parent_order = None  # Parent order ID
+        self.reduce_only = False
+        self.OCO = []  # One-cancels-other
+
         self.data_name = None  # When using custom DataStream
 
         # IB attributes
@@ -134,7 +139,8 @@ class Order:
         self.limit_fee = "0.015"
 
         # CCXT attributes
-        self.ccxt_params = {}
+        self.ccxt_params = {}  # CCXT order parameters
+        self.ccxt_order = {}  # CCXT native order structure
 
         self.reason = None
 
@@ -190,6 +196,11 @@ class Order:
                         return "Invalid order (stop price not provided)"
                     string += f" @ {self.order_stop_price} / {self.order_limit_price}"
 
+                elif self.order_type == "stop":
+                    if self.order_stop_price is None:
+                        return "Invalid order (stop price not provided)"
+                    string += f" @ {self.order_stop_price}"
+
                 return string
 
             else:
@@ -650,6 +661,7 @@ class Trade:
         order_time: datetime,
         order_type: str,
         size: float,
+        last_price: float,
         fill_time: datetime,
         fill_price: float,
         fill_direction: int,
@@ -662,12 +674,17 @@ class Trade:
         self.fill_price = fill_price
         self.direction = fill_direction
         self.fee = fee
+        self.last_price = last_price
         self.order_price = order_price
         self.order_time = order_time
         self.order_type = order_type
         self.size = size
         self.instrument = instrument
 
+        # Precision attributes
+        self._price_precision = 5
+        self._size_precision = 5
+
         # Meta-data
         self.id = None
         self.order_id = None
@@ -722,7 +739,8 @@ class Position:
 
     def __init__(self, **kwargs):
         self.instrument = None
-        self.pnl = None
+        self.net_position = None
+        self.pnl = 0
         self.long_units = None
         self.long_PL = None
         self.long_margin = None
@@ -731,8 +749,26 @@ class Position:
         self.short_margin = None
         self.total_margin = None
         self.trade_IDs = None
-        self.net_position = None
         self.net_exposure = None
+        self.notional = 0
+        self.price_precision = 5
+        self.size_precision = 5
+        self.avg_price = None
+        self._prev_avg_price = None
+
+        # TODO - include position open time, close time
+        # Also need to track max units ? I think that can all be built
+        # from fills
+        self.HCF = 1
+
+        self.entry_time = None
+        self.last_price = None
+        self.last_time = None
+        self.exit_time = None
+        self.exit_price = None
+
+        self.direction = None
+        # self.max_size = None # ?
 
         # IB Attributes
         self.PL = None
@@ -754,6 +790,60 @@ class Position:
     def __str__(self):
         return "AutoTrader Position"
 
+    def _update_with_fill(self, trade: Trade):
+        """Updates the position with the order provided."""
+        net_position_before_fill = self.net_position
+
+        # Update net position
+        self.net_position += trade.size * trade.direction
+
+        # Update average entry price
+        self._prev_avg_price = self.avg_price
+        if self.net_position * net_position_before_fill >= 0:
+            # Position has not flipped
+            if abs(self.net_position) > abs(net_position_before_fill):
+                # Position has increased
+                self.avg_price = round(
+                    (
+                        self.avg_price * abs(self.net_position)
+                        + trade.fill_price * trade.size
+                    )
+                    / (abs(self.net_position) + trade.size),
+                    self.price_precision,
+                )
+            # If position has reduced, average entry price will not change
+        else:
+            # Position has flipped
+            self.avg_price = round(trade.fill_price, self.price_precision)
+
+        # Update last price and last time
+        self.last_price = trade.last_price
+        self.last_time = trade.fill_time
+
+        # TODO - update value of position
+        # self.net_exposure
+        self.notional = self.last_price * abs(self.net_position)
+
+    @classmethod
+    def _from_fill(cls, trade: Trade):
+        """Returns a Position from a fill."""
+        position = cls(
+            instrument=trade.instrument,
+            net_position=abs(trade.size) * trade.direction,
+            last_price=trade.last_price,
+            last_time=trade.fill_time,
+            entry_time=trade.fill_time,
+            entry_price=trade.fill_price,
+            avg_price=trade.fill_price,
+            notional=trade.fill_price * abs(trade.size),
+            price_precision=trade._price_precision,
+            size_precision=trade._size_precision,
+            direction=trade.direction,
+        )
+        # TODO - update attributes created
+        # - exposure, value, etc.
+        return position
+
     def as_dict(self) -> dict:
         """Converts Position object to dictionary.
 
